@@ -93,30 +93,35 @@ class GPUOwner:
     def __init__(self, nb_gpus=1, placeholder_fn=pytorch_placeholder, logger=None, debug_sleep=0.0):
         if logger is None:
             logger = logging
+        self.logger = logger
+        self.debug_sleep = debug_sleep
+        self.placeholder_fn = placeholder_fn
 
         # a workaround for machines where GPU is used also for actual display
         if is_single_gpu_display_mode():
+            logger.info(f"Running on a machine with single GPU used for actual display")
             if nb_gpus == 1:
-                self.placeholders = [placeholder_fn(0)]
-                return
+                self.allocate_gpus(['0'])
             else:
                 raise ValueError(f'Requested {nb_gpus} GPUs on a machine with single one.')
+        else:
+            with SafeUmask(0):  # do not mask any permission out by default, allowing others to work with the lock
+                with open(os.open(LOCK_FILENAME, os.O_CREAT | os.O_WRONLY, 0o666), 'w') as f:
+                    with SafeLock(f, logger):
+                        free_gpus = get_free_gpus()
+                        if len(free_gpus) < nb_gpus:
+                            raise RuntimeError(f"Required {nb_gpus} GPUs, only found these free: {free_gpus}. Somebody didn't properly declare their resources?")
+                        gpus_to_allocate = free_gpus[:nb_gpus]
+                        self.allocate_gpus(gpus_to_allocate)
 
-        with SafeUmask(0):  # do not mask any permission out by default, allowing others to work with the lock
-            with open(os.open(LOCK_FILENAME, os.O_CREAT | os.O_WRONLY, 0o666), 'w') as f:
-                with SafeLock(f, logger):
-                    free_gpus = get_free_gpus()
-                    if len(free_gpus) < nb_gpus:
-                        raise RuntimeError(f"Required {nb_gpus} GPUs, only found these free: {free_gpus}. Somebody didn't properly declare their resources?")
-                    gpus_to_allocate = free_gpus[:nb_gpus]
+    def allocate_gpus(self, gpu_device_numbers):
+        os.environ['CUDA_VISIBLE_DEVICES'] = ','.join(gpu_device_numbers)
+        self.logger.info(f"Set CUDA_VISIBLE_DEVICES={os.environ['CUDA_VISIBLE_DEVICES']}")
 
-                    os.environ['CUDA_VISIBLE_DEVICES'] = ','.join(gpus_to_allocate)
-                    logger.info(f"Set CUDA_VISIBLE_DEVICES={os.environ['CUDA_VISIBLE_DEVICES']}")
+        time.sleep(self.debug_sleep)
 
-                    time.sleep(debug_sleep)
-
-                    try:
-                        self.placeholders = [placeholder_fn(device_no) for device_no in range(nb_gpus)]
-                    except RuntimeError:
-                        logger.error('Failed to acquire placeholder, truly marvellous. Race condition with someone not using `GPUOwner?`')
-                        raise
+        try:
+            self.placeholders = [self.placeholder_fn(device_no) for device_no in range(len(gpu_device_numbers))]
+        except RuntimeError:
+            self.logger.error('Failed to acquire placeholder, truly marvellous. Race condition with someone not using `GPUOwner?`')
+            raise
